@@ -29,6 +29,16 @@ xmlns = 'http://pythonhosted.org/genemail/xmlns/1.0'
 htmlns = util.htmlns
 class MissingHeader(Exception): pass
 
+#------------------------------------------------------------------------------
+class AutoCachingDict(dict):
+  def get(self, key, default=None):
+    if key in self:
+      return self[key]
+    if callable(default):
+      default = default()
+    self[key] = default
+    return default
+
 #------------------------------------------------------------------------------
 class Email(object):
 
@@ -56,12 +66,12 @@ class Email(object):
           {'mime:related; type="text/html"': [
             'email:html',
             {'mime:attachments': 'email:attachments; cid=True'},
-            ]},
           ]},
+        ]},
         {'mime:attachments': 'email:attachments; cid=False'},
-        ]
+      ]
 
-      },
+    },
     params               = {},
     attachments          = [],
     headers              = idict(),
@@ -74,7 +84,7 @@ class Email(object):
     encoding             = None,
     boundary             = None,
     minimalMime          = True,
-    )
+  )
 
   #----------------------------------------------------------------------------
   def __init__(self, manager, name, provider=None, default=None):
@@ -87,7 +97,7 @@ class Email(object):
       default = adict(self.DEFAULTS)
     for attr in self.DEFAULTS.keys():
       setattr(self, attr, copy.deepcopy(getattr(default, attr, None)))
-
+    self.params['cache'] = AutoCachingDict()
     # self.attachmentTable    = None
     # for key, val in self.provider.getMap('headers', {}).items():
     #   self.headers[key] = val
@@ -223,7 +233,7 @@ class Email(object):
         contentType = node.get('content-type', None),
         value       = node.get('value', None) or node.text,
         cid         = node.get('cid', 'false').lower() == 'true',
-        )
+      )
       if node.get('encoding', None) == 'base64':
         att.value = base64.b64decode(att.value)
       atts[att.name] = att
